@@ -1,7 +1,7 @@
 /*
  * PCRE - Perl Compatible Regular Expressions
  * <http://www.pcre.org/>
- * Copyright (C) 2003-2007 Sam Steingold
+ * Copyright (C) 2003-2008 Sam Steingold
  * GPL2
  */
 
@@ -23,9 +23,7 @@
 #error to CFLAGS make variable and rebuild CLISP before coming back here.
 #endif
 
-#if defined(STDC_HEADERS)
-# include <string.h>            /* for memcpy(3) */
-#endif
+#include <string.h>             /* for memcpy(3) */
 
 DEFMODULE(pcre,"PCRE")
 
@@ -80,9 +78,9 @@ DEFCHECKER(pcre_options,prefix=PCRE,bitmasks=both,\
            CASELESS MULTILINE DOTALL EXTENDED ANCHORED DOLLAR-ENDONLY EXTRA \
            NOTBOL NOTEOL UNGREEDY NOTEMPTY UTF8 NO-AUTO-CAPTURE NO-UTF8-CHECK \
            AUTO-CALLOUT PARTIAL)
-DEFUN(PCRE:PCRE-COMPILE,string &key :STUDY :IGNORE-CASE :MULTILINE :DOTALL \
-      :EXTENDED :ANCHORED :DOLLAR-ENDONLY :EXTRA :NOTBOL :NOTEOL :UNGREADY \
-      :NOTEMPTY :NO-AUTO-CAPTURE :AUTO-CALLOUT :PARTIAL)
+DEFUN(PCRE:PCRE-COMPILE,string &key STUDY IGNORE-CASE MULTILINE DOTALL \
+      EXTENDED ANCHORED DOLLAR-ENDONLY EXTRA NOTBOL NOTEOL UNGREADY \
+      NOTEMPTY NO-AUTO-CAPTURE AUTO-CALLOUT PARTIAL)
 { /* compile the pattern, return PATTERN struct */
   int options = PCRE_UTF8 | pcre_compile_flags();
   bool study = !missingp(STACK_0);
@@ -145,7 +143,7 @@ static void check_pattern (gcv_object_t *pat, pcre** compiled_pattern,
 DEFCHECKER(error_pcre_code,prefix=PCRE_ERROR, NOMATCH NULL BADOPTION    \
            BADMAGIC UNKNOWN_NODE NOMEMORY NOSUBSTRING MATCHLIMIT CALLOUT \
            DFA_UITEM DFA_UCOND DFA_UMLIMIT DFA_WSSIZE DFA_RECURSE       \
-           BADUTF8 BADUTF8_OFFSET PARTIAL BADPARTIAL INTERNAL BADCOUNT)
+           BADUTF8 BADUTF8_OFFSET PARTIAL BADPARTIAL :INTERNAL BADCOUNT)
 nonreturning_function(static, error_pcre, (int status)) {
   pushSTACK(error_pcre_code_reverse(status));
   pushSTACK(sfixnum(status)); pushSTACK(TheSubr(subr_self)->name);
@@ -159,7 +157,7 @@ nonreturning_function(static, error_pcre, (int status)) {
     end_system_call();                                  \
     if (status < bad) error_pcre(status);               \
   } while(0)
-DEFCHECKER(fullinfo_arg,prefix=PCRE_INFO,OPTIONS SIZE CAPTURECOUNT      \
+DEFCHECKER(fullinfo_arg,prefix=PCRE_INFO,OPTIONS :SIZE CAPTURECOUNT     \
            BACKREFMAX FIRSTBYTE FIRSTTABLE LASTLITERAL                  \
            NAMEENTRYSIZE NAMECOUNT NAMETABLE STUDYSIZE)
 /* PCRE_INFO_DEFAULTTABLES -- does not look useful
@@ -242,7 +240,7 @@ DEFUN(PCRE:PATTERN-INFO,pattern &optional request)
   if (missingp(STACK_0)) {
     int count = 0;
     pushSTACK(`:OPTIONS`); pushSTACK(fullinfo_options(c_pat,study)); count+=2;
-    pushSTACK(`:SIZE`);
+    pushSTACK(S(Ksize));
     pushSTACK(fullinfo_size(c_pat,study,PCRE_INFO_SIZE)); count+=2;
     pushSTACK(`:CAPTURECOUNT`);
     pushSTACK(fullinfo_int(c_pat,study,PCRE_INFO_CAPTURECOUNT)); count+=2;
@@ -337,8 +335,8 @@ DEFUN(PCRE:PCRE-NAME-TO-INDEX,pattern name)
 
 DEFFLAGSET(pcre_exec_flags, PCRE_ANCHORED PCRE_NOTBOL PCRE_NOTEOL \
            PCRE_NOTEMPTY PCRE_PARTIAL PCRE_DFA_SHORTEST PCRE_DFA_RESTART)
-DEFUN(PCRE:PCRE-EXEC,pattern subject &key :WORK-SPACE :DFA :BOOLEAN :OFFSET \
-      :ANCHORED :NOTBOL :NOTEOL :NOTEMPTY :PARTIAL :DFA-SHORTEST :DFA-RESTART)
+DEFUN(PCRE:PCRE-EXEC,pattern subject &key WORK-SPACE DFA BOOLEAN OFFSET \
+      ANCHORED NOTBOL NOTEOL NOTEMPTY PARTIAL DFA-SHORTEST DFA-RESTART)
 { /* match the SUBJECT string against a pre-compiled PATTERN;
      return a vector of MATCH structures or NIL if no matches */
   int options = pcre_exec_flags();
