@@ -1,4 +1,4 @@
-;;; Copyright (C) 2000-2006 by Sam Steingold
+;;; Copyright (C) 2000-2006, 2008 by Sam Steingold
 ;;; This file is a part of CLISP (http://clisp.cons.org), and, as such,
 ;;; is distributed under the GNU GPL (http://www.gnu.org/copyleft/gpl.html)
 
@@ -44,7 +44,7 @@
 (defun read-from-file (file &key (out *standard-output*) (package "KEYWORD")
                        repeat)
   "Read an object from a file.
-The keyword argument KEYWORD specifies the package to read in.
+The keyword argument PACKAGE specifies the package to read in.
 The keyword argument OUT specifies the output for log messages.
 The keyword argument REPEAT specifies how many objects to read:
  If NIL (default), read once and return the object read;
@@ -66,7 +66,7 @@ The keyword argument REPEAT specifies how many objects to read:
                    internal-time-units-per-second))))))
 
 (defun browse-url (url &key (browser *browser*) (out *standard-output*))
-  "Run the browser (a keyword in `*browsers*' or a list) on the URL."
+  "Run the browser (a keyword in *BROWSERS* or a list) on the URL."
   #+WIN32
   (when (eq browser :default) ; feed url to ShellExecute
     (when out
@@ -92,10 +92,11 @@ The keyword argument REPEAT specifies how many objects to read:
           (t (format t "~s: no browser specified; please point your browser at
  --> <URL:~a>~%" 'browse-url url)))))
 
-;;; see also clocc/cllib/net.lisp
+;;; keep in sync with clocc/cllib/url.lisp
 (defvar *http-proxy* nil
   "A list of 3 elements (user:password host port), parsed from $http_proxy
-proxy-user:proxy-password@proxy-host:proxy-port")
+\[http://]proxy-user:proxy-password@proxy-host:proxy-port[/]
+by HTTP-PROXY.")
 (defconstant *http-port* 80)
 (defun http-proxy (&optional (proxy-string (getenv "http_proxy") proxy-p))
   "When the argument is supplied or *HTTP-PROXY* is NIL, parse the argument,
@@ -107,12 +108,13 @@ set *HTTP-PROXY*, and return it; otherwise just return *HTTP-PROXY*."
                                                #2=#.(length #1#)))
                       #2# 0))
            (at (position #\@ proxy-string :start start))
-           (colon (position #\: proxy-string :start (or at start))))
+           (colon (position #\: proxy-string :start (or at start)))
+           (slash (position #\/ proxy-string :start (or colon at start))))
       (setq *http-proxy*
             (list (and at (subseq proxy-string start at))
-                  (subseq proxy-string (if at (1+ at) start) colon)
+                  (subseq proxy-string (if at (1+ at) start) (or colon slash))
                   (if colon
-                      (parse-integer proxy-string :start (1+ colon))
+                      (parse-integer proxy-string :start (1+ colon) :end slash)
                       *http-port*)))
       (format t "~&;; ~S=~S~%" '*http-proxy* *http-proxy*)))
   *http-proxy*)
@@ -334,7 +336,11 @@ set *HTTP-PROXY*, and return it; otherwise just return *HTTP-PROXY*."
   (let ((pack (symbol-package obj)))
     ;; do not search impnotes for user symbols
     (when (and pack
-               (member (package-name pack) *system-package-list* :test #'equal)
+               (or (eq pack #,(find-package "CUSTOM"))
+                   ;; CUSTOM is not in *SYSTEM-PACKAGE-LIST*
+                   ;; because it must not be locked
+                   (member (package-name pack) *system-package-list*
+                           :test #'equal))
                (ensure-impnotes-map))
       (let ((doc (call-next-method)))
         (if doc
