@@ -5,20 +5,22 @@
 (defun show (object &key ((:pretty *print-pretty*) *print-pretty*))
   "Print the object on its own line and return it. Used in many tests!"
   (fresh-line) (prin1 object) (terpri) object)
+#+clisp (progn
 (defun kill-down (name)
   (dolist (f (directory (ext:string-concat name "**")))
     (format t "~&removing ~S~%" f)
     (if (pathname-name f)
         (delete-file f)
-        (ext:delete-dir f))))
+        (ext:delete-directory f))))
 (defun rmrf (name)
   (ext:dir (ext:string-concat name "**"))
   (kill-down name)
   (format t "~&removing ~S~%" name)
-  (ext:delete-dir name))
-(defun prepare-dir (name)
+  (ext:delete-directory name))
+(defun prepare-directory (name)
   (ensure-directories-exist name :verbose t)
   (kill-down name))
+)
 (defun show-file (file)         ; return line count
   (with-open-file (st file :direction :input)
     (format t "~&~S: ~:D byte~:P:~%" file (file-length st))
@@ -364,8 +366,7 @@ NIL: sacla-style: forms should evaluate to non-NIL.")
                   #+(or CLISP ALLEGRO CMU19 OpenMCL LISPWORKS) "weakhash"
                   #+(or CLISP LISPWORKS) "weakhash2"))
       (push (run-test ff) res))
-    (unless disable-risky ; fails on amd64 - disable for now...
-      (push (run-test "bind" :eval-method :eval :logname "bind-eval") res))
+    (push (run-test "bind" :eval-method :eval :logname "bind-eval") res)
     (push (run-test "bind" :eval-method :compile :logname "bind-compile") res)
     #+(or CLISP ALLEGRO CMU SBCL LISPWORKS)
     (let ((tmp (list "weakptr" 0 0)))
